/*
 * Copyright (C) 2010 David Arkell.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.gwtorbit.client.model;

import java.util.Iterator;

import com.google.gwt.user.client.ui.HasWidgets;
import com.google.gwt.user.client.ui.Widget;

public class DisplayMethods {

	public static void displayData(Object widget, UiData uiData) {
		if (widget instanceof UiDataDisplay) {
			UiDataDisplay uiDataDisplay = (UiDataDisplay) widget;
			// if (!uiDataDisplay.getIgnore()) {
			uiDataDisplay.displayData(uiData);
			// }
		} else if (widget instanceof HasWidgets) {
			HasWidgets hasWidgets = (HasWidgets) widget;

			Iterator<Widget> widgets = hasWidgets.iterator();
			while (widgets.hasNext()) {
				displayData(widgets.next(), uiData);
			}
		}
	}

	public static void clearErrors(Object widget) {
		if (widget instanceof UiErrorDisplay) {
			UiErrorDisplay uiErrorDisplay = (UiErrorDisplay) widget;
			uiErrorDisplay.clearErrors();
		} else if (widget instanceof HasWidgets) {
			HasWidgets hasWidgets = (HasWidgets) widget;

			Iterator<Widget> widgets = hasWidgets.iterator();
			while (widgets.hasNext()) {
				clearErrors(widgets.next());
			}
		}
	}

	public static void clearDisplay(Object widget) {
		if (widget instanceof UiDataDisplay) {
			UiDataDisplay uiDataDisplay = (UiDataDisplay) widget;
			uiDataDisplay.clearData();
		} else if (widget instanceof HasWidgets) {
			HasWidgets hasWidgets = (HasWidgets) widget;

			Iterator<Widget> widgets = hasWidgets.iterator();
			while (widgets.hasNext()) {
				clearDisplay(widgets.next());
			}
		}
	}

	public static void displayErrors(Object widget, UiErrors errors) {
		displayErrors(widget, errors, "");
	}

	public static void displayErrors(Object widget, UiErrors errors, String errorPrefix) {
		if (widget instanceof UiErrorDisplay) {
			UiErrorDisplay uiErrorDisplay = (UiErrorDisplay) widget;
			uiErrorDisplay.displayErrors(errors, errorPrefix);
		} else if (widget instanceof HasWidgets) {
			HasWidgets hasWidgets = (HasWidgets) widget;

			Iterator<Widget> widgets = hasWidgets.iterator();
			while (widgets.hasNext()) {
				displayErrors(widgets.next(), errors, errorPrefix);
			}
		}
	}

	public static void getDataAndShowErrors(Object widget, UiData uiData, UiErrors errors) {
		clearErrors(widget);
		getData(widget, uiData, errors, "");
		if (errors.hasErrors()) {
			displayErrors(widget, errors, "");
		}
	}

	public static void getData(Object widget, UiData uiData, UiErrors errors) {
		getData(widget, uiData, errors, "");
	}

	public static void getData(Object widget, UiData uiData, UiErrors errors, String errorPrefix) {
		getDataInt(widget, uiData, errors, errorPrefix);

	}

	private static void getDataInt(Object widget, UiData uiData, UiErrors errors, String errorPrefix) {
		if (widget instanceof UiDataDisplay) {
			UiDataDisplay uiDataDisplay = (UiDataDisplay) widget;
			if (!uiDataDisplay.getIgnore()) {
				uiDataDisplay.getData(uiData, errors, errorPrefix);
			}
		} else if (widget instanceof HasWidgets) {
			HasWidgets hasWidgets = (HasWidgets) widget;

			Iterator<Widget> widgets = hasWidgets.iterator();
			while (widgets.hasNext()) {
				getDataInt(widgets.next(), uiData, errors, errorPrefix);
			}
		}
	}

}
