/*
 * Copyright (C) 2010 David Arkell.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.gwtorbit.client.widgets;

import com.google.gwt.user.client.ui.HorizontalPanel;
import com.google.gwt.user.client.ui.SimplePanel;
import com.google.gwt.user.client.ui.Widget;

public abstract class FormAbstractRow extends HorizontalPanel {

	private SimplePanel leftPanel;

	private LLabel label;

	private SimplePanel fieldPanel;

	public FormAbstractRow() {
		setStyleName("UiFormRow");

		leftPanel = new SimplePanel();
		leftPanel.setStyleName("UiFormFieldLeft");
		super.add(leftPanel);

		label = new LLabel();
		label.setStyleName("UiFormFieldLabel");
		super.add(label);

		fieldPanel = new SimplePanel();
		fieldPanel.setStyleName("UiFormField");
		super.add(fieldPanel);
	}

	protected void setFieldWidget(Widget w) {
		fieldPanel.setWidget(w);
	}

	public void setLabelText(String labelText) {
		label.setText(labelText);
	}

	public void setLabelKey(String labelKey) {
		label.setKey(labelKey);
	}

	public void setMandatory(boolean mandatory) {
		this.mandatory = mandatory;
		if (mandatory) {
			label.addStyleName("UiFormFieldLabelMand");
		} else {
			label.removeStyleName("UiFormFieldLabelMand");
		}
	}

	public boolean isMandatory() {
		return mandatory;
	}

	private boolean mandatory;

	public LLabel getFormLabel() {
		return label;
	}

	public SimplePanel getLeftPanel() {
		return leftPanel;
	}
}
