/*
 * Copyright (C) 2010 David Arkell.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.gwtorbit.client.widgets;

import java.util.ArrayList;

import com.google.gwt.dom.client.Document;
import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.dom.client.ClickHandler;
import com.google.gwt.event.dom.client.DomEvent;
import com.google.gwt.event.dom.client.HasClickHandlers;
import com.google.gwt.event.shared.HandlerRegistration;
import com.google.gwt.user.client.ui.RadioButton;
import com.google.gwt.user.client.ui.VerticalPanel;
import com.google.gwt.user.client.ui.Widget;
import com.gwtorbit.client.model.UiData;
import com.gwtorbit.client.model.UiDataDisplay;
import com.gwtorbit.client.model.UiErrorDisplay;
import com.gwtorbit.client.model.UiErrors;
import com.gwtorbit.client.model.UiLocalizedError;
import com.gwtorbit.client.mvp.ClientResourceBundle;

public class UiListOfRadioButtons extends VerticalPanel implements HasClickHandlers, UiDataDisplay,
		UiErrorDisplay {

	private static int counter = 0;

	public UiListOfRadioButtons() {
		radioButtonName = "UiListOfRadioButtons" + (counter++);
	}

	private String radioButtonName;

	private ArrayList<Object> optionsData = new ArrayList<Object>();

	public void displayData(UiData uiData) {
		Object data = uiData.getData(dataName);
		if (data != null) {
			for (int i = 0; i < optionsData.size(); i++) {
				if (optionsData.get(i).equals(data)) {
					RadioButton rb = (RadioButton) this.getWidget(i);
					rb.setValue(true);
					break;
				}
			}
		}
	}

	public Object getData() {
		Object data = null;
		for (int i = 0; i < optionsData.size(); i++) {
			RadioButton rb = (RadioButton) this.getWidget(i);
			if (rb.getValue()) {
				data = optionsData.get(i);
				break;
			}
		}
		return data;
	}

	public void getData(UiData uiData, UiErrors errors, String errorPrefix) {
		Object data = getData();
		if (mandatory && data == null) {
			errors.addError(errorPrefix + dataName, UiLocalizedError.MANDATORY);
		}
		uiData.setData(dataName, data);
	}

	private boolean mandatory = false;

	private String dataName;

	public String getDataName() {
		return dataName;
	}

	public void setDataName(String dataName) {
		this.dataName = dataName;
	}

	public void setMandatory(boolean mandatory) {
		this.mandatory = mandatory;
	}

	public boolean isMandatory() {
		return mandatory;
	}

	public void clearErrors() {
		this.removeStyleName("UiTextBoxError");
	}

	public void displayErrors(UiErrors errors, String errorPrefix) {
		if (errors.hasErrors(errorPrefix + dataName)) {
			this.addStyleName("UiTextBoxError");
		}
	}

	public void clearData() {
		for (int i = 0; i < getWidgetCount(); i++) {
			RadioButton radioButton = (RadioButton) this.getWidget(i);
			radioButton.setValue(false);
		}
	}

	public void add(Widget w) {
		if (w instanceof UiOption) {
			UiOption uiOption = (UiOption) w;

			String label = uiOption.getLabel();
			if (uiOption.getLabelKey() != null) {
				label = ClientResourceBundle.getString(uiOption.getLabelKey());
			}
			this.addOption(label, uiOption.getValue());
		}
	}

	public void addOption(String label, Object value) {
		optionsData.add(value);
		RadioButton rb = new RadioButton(radioButtonName, label);
		super.add(rb);
	}

	private boolean ignore = false;

	public boolean getIgnore() {
		return ignore;
	}

	public void setIgnore(boolean ignore) {
		this.ignore = ignore;
	}

	public void clear() {
		optionsData.clear();
		super.clear();
	}

	public void clearOptions() {
		clear();
	}

	public HandlerRegistration addClickHandler(final ClickHandler handler) {
		// TODO: this needs to be better.
		for (int i = 0; i < getWidgetCount(); i++) {
			RadioButton radioButton = (RadioButton) this.getWidget(i);
			radioButton.addClickHandler(new ClickHandler() {
				public void onClick(ClickEvent event) {
					DomEvent.fireNativeEvent(Document.get().createClickEvent(0, 0, 0, 0, 0,
							event.getNativeEvent().getCtrlKey(),
							event.getNativeEvent().getAltKey(),
							event.getNativeEvent().getShiftKey(),
							event.getNativeEvent().getMetaKey()), UiListOfRadioButtons.this);
				}
			});
		}

		return this.addDomHandler(new ClickHandler() {
			public void onClick(ClickEvent event) {
				handler.onClick(event);
			}
		}, ClickEvent.getType());
	}

}
