/*
 * Copyright (C) 2010 David Arkell.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.gwtorbit.client.widgets;

import com.google.gwt.user.client.ui.TextBox;
import com.gwtorbit.client.model.UiData;
import com.gwtorbit.client.model.UiDataDisplay;
import com.gwtorbit.client.model.UiErrorDisplay;
import com.gwtorbit.client.model.UiErrors;
import com.gwtorbit.client.model.UiLocalizedError;
import com.gwtorbit.client.mvp.ClientResourceBundle;

public class UiTextBoxInteger extends TextBox implements UiDataDisplay, UiErrorDisplay {

	public UiTextBoxInteger() {
		super();
		setFieldSize("short");
	}

	public void displayData(UiData uiData) {
		this.setInteger((Integer) uiData.getData(dataName));
	}

	public void getData(UiData uiData, UiErrors errors, String errorPrefix) {

		Integer integer = null;

		try {
			integer = this.getInteger();
			if (mandatory && integer == null) {
				errors.addError(errorPrefix + dataName, UiLocalizedError.MANDATORY);
			}
		} catch (NumberFormatException nfe) {
			errors.addError(errorPrefix + dataName, UiLocalizedError.INVALID);
		}

		uiData.setData(dataName, integer);
	}

	private boolean mandatory = false;

	private String dataName;

	public String getDataName() {
		return dataName;
	}

	public void setDataName(String dataName) {
		this.dataName = dataName;
	}

	public void setMandatory(boolean mandatory) {
		this.mandatory = mandatory;
	}

	public boolean isMandatory() {
		return mandatory;
	}

	public void clearErrors() {
		this.removeStyleName("UiTextBoxError");
	}

	public void displayErrors(UiErrors errors, String errorPrefix) {
		if (errors.hasErrors(errorPrefix + dataName)) {
			this.addStyleName("UiTextBoxError");
		}
	}

	public void clearData() {
		setText("");
	}

	private boolean ignore = false;

	public boolean getIgnore() {
		return ignore;
	}

	public void setIgnore(boolean ignore) {
		this.ignore = ignore;
	}

	public void setInteger(Integer value) {
		if (value == null) {
			setValue("");
		} else {
			setValue(ClientResourceBundle.integerFormat.format(value.doubleValue()));
		}
	}

	public Integer getInteger() throws NumberFormatException {
		if ("".equals(getValue())) {
			return null;
		}
		double value = ClientResourceBundle.integerFormat.parse(getValue());
		return (int) value;
	}

	private String fieldSize;

	public void setFieldSize(String size) {
		if ("short".equalsIgnoreCase(size)) {
			fieldSize = "short";
			setStyleName("UiTextBoxShort");
		} else if ("long".equalsIgnoreCase(size)) {
			fieldSize = "long";
			setStyleName("UiTextBoxLong");
		} else {
			fieldSize = "medium";
			setStyleName("UiTextBoxMedium");
		}
	}

	public String getFieldSize() {
		return fieldSize;
	}

}
